//--------------------------------------------------------------
// File     : stm32_ub_ws2812.h
//--------------------------------------------------------------

//--------------------------------------------------------------
#ifndef __STM32F4_UB_WS2812_H
#define __STM32F4_UB_WS2812_H


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include <stdint.h>
#include "stm32f10x_gpio.h"
#include "stm32f10x_rcc.h"
#include "stm32f10x_tim.h"
#include "stm32f10x_dma.h"
#include "stm32f10x_exti.h"
#include "misc.h"

//--------------------------------------------------------------
// Struktur der WS2812-Ketten laengen
// (wird nach PowerOn aktuallisiert)
//--------------------------------------------------------------
typedef struct {
  uint32_t ch1;      // anzahl der LEDs an CH1
  uint32_t ch2;      // anzahl der LEDs an CH2
  uint32_t ch3;      // anzahl der LEDs an CH3
  uint32_t ch4;      // anzahl der LEDs an CH4
} WS2812_CHAIN_LEN_t;

extern WS2812_CHAIN_LEN_t WS2812_CHAIN_LEN;

//--------------------------------------------------------------
// max Anzahl LEDs in einer WS2812-Kette
//--------------------------------------------------------------
#define  WS2812_LED_MAX_ANZ    10

//--------------------------------------------------------------
// Enable/Disable der einzelnen Ketten
// 1=enable, 0=disable
//--------------------------------------------------------------
#define  WS2812_CH1_ENABLE   1  // [CH1 an PA6]
#define  WS2812_CH2_ENABLE   1  // [CH2 an PA2]
#define  WS2812_CH3_ENABLE   1  // [CH3 an PB0]
#define  WS2812_CH4_ENABLE   1  // [CH4 an PB1]

// benutzer Timer fuer das Daten-Signal => TIM3
//--------------------------------------------------------------
#define  WS2812_TIM_CLOCK     RCC_APB1Periph_TIM3
#define  WS2812_TIM_CLOCK_CH2 RCC_APB1Periph_TIM2
#define  WS2812_TIM           TIM3
#define  WS2812_TIM_AF        GPIO_AF_TIM3
#define  WS2812_TIM_CH1       1 
#define  WS2812_TIM_CCR_REG1  TIM3->CCR1
#define  WS2812_TIM_DMA_TRG1  TIM_DMA_CC1
#define  WS2812_TIM_CH2       TIM2 
#define  WS2812_TIM_CCR_REG2  TIM2->CCR2
#define  WS2812_TIM_DMA_TRG2  TIM_DMA_CC2
#define  WS2812_TIM_CH3       3 
#define  WS2812_TIM_CCR_REG3  TIM3->CCR3
#define  WS2812_TIM_DMA_TRG3  TIM_DMA_CC3
#define  WS2812_TIM_CH4       4 
#define  WS2812_TIM_CCR_REG4  TIM3->CCR4
#define  WS2812_TIM_DMA_TRG4  TIM_DMA_CC4



//--------------------------------------------------------------
// GPIO-Pins (CH1...CH4) fuer Data-OUT 
//
// moegliche Pinbelegungen (bei TIM3)
//   CH1 : [PA6, PB4, PC6]
//   CH2 : [PA7, PB5, PC7]
//   CH3 : [PB0, PC8]
//   CH4 : [PB1, PC9] 
//--------------------------------------------------------------
#define  WS2812_CH1_CLOCK    RCC_APB2Periph_GPIOA
#define  WS2812_CH1_PORT     GPIOA
#define  WS2812_CH1_PIN      GPIO_Pin_6 
#define  WS2812_CH1_SOURCE   GPIO_PinSource6 

#define  WS2812_CH2_CLOCK    RCC_APB2Periph_GPIOA
#define  WS2812_CH2_PORT     GPIOA
#define  WS2812_CH2_PIN      GPIO_Pin_1
#define  WS2812_CH2_SOURCE   GPIO_PinSource1

#define  WS2812_CH3_CLOCK    RCC_APB2Periph_GPIOB
#define  WS2812_CH3_PORT     GPIOB
#define  WS2812_CH3_PIN      GPIO_Pin_0 
#define  WS2812_CH3_SOURCE   GPIO_PinSource0

#define  WS2812_CH4_CLOCK    RCC_APB2Periph_GPIOB
#define  WS2812_CH4_PORT     GPIOB
#define  WS2812_CH4_PIN      GPIO_Pin_1 
#define  WS2812_CH4_SOURCE   GPIO_PinSource1

//--------------------------------------------------------------
// GPIO-Pins fuer EXTI
// (ein PIN gemeinsam fuer alle LED-Ketten)
//--------------------------------------------------------------
#define  WS2812_EXTI_CLOCK   RCC_APB2Periph_GPIOB
#define  WS2812_EXTI_PORT    GPIOB
#define  WS2812_EXTI_PIN     GPIO_Pin_8
#define  WS2812_EXTI_SOURCE  EXTI_PortSourceGPIOB

//--------------------------------------------------------------
// benutzer DMA
//   => TIM3-CC1 => DMA1, Channel6
//   => TIM2-CC2 => DMA1, Channel7 (!!!)
//   => TIM3-CC3 => DMA1, Channel2
//   => TIM3-CC4 => DMA1, Channel3
//--------------------------------------------------------------
#define  WS2812_DMA_CLOCK         RCC_AHBPeriph_DMA1

//#define  WS2812_DMA_CH1_STREAM    DMA1_Stream6
#define  WS2812_DMA_CH1_CHANNEL   DMA1_Channel6

//#define  WS2812_DMA_CH2_STREAM    DMA1_Stream1
#define  WS2812_DMA_CH2_CHANNEL   DMA1_Channel7

//#define  WS2812_DMA_CH3_STREAM    DMA1_Stream2
#define  WS2812_DMA_CH3_CHANNEL   DMA1_Channel2

//#define  WS2812_DMA_CH4_STREAM    DMA1_Stream3
#define  WS2812_DMA_CH4_CHANNEL   DMA1_Channel3

//--------------------------------------------------------------
// Transfer-Complete Interrupt
//   CC1 => DMA1, Channel6
//   CC2 => DMA1, Channel7 (TIM2_CH2)
//   CC3 => DMA1, Channel2
//   CC4 => DMA1, Channel3
//--------------------------------------------------------------
#define  WS2812_DMA_CH1_IRQn      DMA1_Channel6_IRQn
#define  WS2812_DMA_CH1_ISR       DMA1_Channel6_IRQHandler
#define  WS2812_DMA_CH1_IRQ_FLAG  DMA1_IT_TC6

#define  WS2812_DMA_CH2_IRQn      DMA1_Channel7_IRQn
#define  WS2812_DMA_CH2_ISR       DMA1_Channel7_IRQHandler
#define  WS2812_DMA_CH2_IRQ_FLAG  DMA1_IT_TC7

#define  WS2812_DMA_CH3_IRQn      DMA1_Channel2_IRQn
#define  WS2812_DMA_CH3_ISR       DMA1_Channel2_IRQHandler
#define  WS2812_DMA_CH3_IRQ_FLAG  DMA1_IT_TC2

#define  WS2812_DMA_CH4_IRQn      DMA1_Channel3_IRQn
#define  WS2812_DMA_CH4_ISR       DMA1_Channel3_IRQHandler
#define  WS2812_DMA_CH4_IRQ_FLAG  DMA1_IT_TC3

//--------------------------------------------------------------
// RGB LED Farbdefinition (3 x 8bit)
//--------------------------------------------------------------
typedef struct {
  uint8_t red;    // 0...255 (als PWM-Wert)
  uint8_t green;  // 0...255  (als PWM-Wert)
  uint8_t blue;   // 0...255 (als PWM-Wert)
}WS2812_RGB_t;


//--------------------------------------------------------------
// HSV LED Farbdefinition
//--------------------------------------------------------------
typedef struct {
  uint16_t h;     // 0...359 (in Grad, 0=R, 120=G, 240=B)
  uint8_t s;      // 0...100 (in Prozent)
  uint8_t v;      // 0...100 (in Prozent)
}WS2812_HSV_t;

//--------------------------------------------------------------
// Globale Buffer fuer die Farben (als RGB-Wert)
//--------------------------------------------------------------
//#if WS2812_CH1_ENABLE>0
  extern WS2812_RGB_t WS2812_LED_BUF_CH1[WS2812_LED_MAX_ANZ];
//#endif

//#if WS2812_CH2_ENABLE>0
  extern WS2812_RGB_t WS2812_LED_BUF_CH2[WS2812_LED_MAX_ANZ];
//#endif

//#if WS2812_CH3_ENABLE>0
  extern WS2812_RGB_t WS2812_LED_BUF_CH3[WS2812_LED_MAX_ANZ];
//#endif

//#if WS2812_CH4_ENABLE>0
  extern WS2812_RGB_t WS2812_LED_BUF_CH4[WS2812_LED_MAX_ANZ];
//#endif

//--------------------------------------------------------------
// standard Farben (R,G,B)
//--------------------------------------------------------------
#define  WS2812_RGB_COL_OFF      (WS2812_RGB_t) {0x00,0x00,0x00}

#define  WS2812_RGB_COL_BLUE     (WS2812_RGB_t) {0x00,0x00,0xFF}
#define  WS2812_RGB_COL_GREEN    (WS2812_RGB_t) {0x00,0xFF,0x00}
#define  WS2812_RGB_COL_RED      (WS2812_RGB_t) {0xFF,0x00,0x00}
#define  WS2812_RGB_COL_WHITE    (WS2812_RGB_t) {0xFF,0xFF,0xFF}

#define  WS2812_RGB_COL_CYAN     (WS2812_RGB_t) {0x00,0xFF,0xFF}
#define  WS2812_RGB_COL_MAGENTA  (WS2812_RGB_t) {0xFF,0x00,0xFF}
#define  WS2812_RGB_COL_YELLOW   (WS2812_RGB_t) {0xFF,0xFF,0x00}

//--------------------------------------------------------------
// standard Farben (H,S,V)
//--------------------------------------------------------------
#define  WS2812_HSV_COL_OFF      (WS2812_HSV_t) {0,  0,  0}

#define  WS2812_HSV_COL_BLUE     (WS2812_HSV_t) {240,100,100}
#define  WS2812_HSV_COL_GREEN    (WS2812_HSV_t) {120,100,100}
#define  WS2812_HSV_COL_RED      (WS2812_HSV_t) {0,  100,100}

#define  WS2812_HSV_COL_CYAN     (WS2812_HSV_t) {180,100,100}
#define  WS2812_HSV_COL_MAGENTA  (WS2812_HSV_t) {300,100,100}
#define  WS2812_HSV_COL_YELLOW   (WS2812_HSV_t) {60, 100,100}

//--------------------------------------------------------------
// WS2812 Timing : (1.25us = 800 kHz)
//   logische-0 => HI:0.35us , LO:0.90us
//   logische-1 =  HI:0.90us , LO:0.35us
//
// WS23812 Bit-Format : (8G8R8B)
//   24bit pro LED  (30us pro LED)
//    8bit pro Farbe (MSB first)
//    Farbreihenfolge : GRB
//      
//   nach jedem Frame von n-LEDs kommt eine Pause von >= 50us
// 
// STM32F4xx
// Grundfrequenz (TIM3) = 2*APB1 (APB1=42MHz) => TIM_CLK=84MHz
// periode   : 0 bis 0xFFFF
// prescale  : 0 bis 0xFFFF
//
// PWM-Frq = TIM_CLK/(periode+1)/(vorteiler+1)
//
//-------------------------------------------------------------- 
// STM32F4xx
//#define  WS2812_TIM_PRESCALE    0  // F_T3  = 84 MHz (11.9ns)
//#define  WS2812_TIM_PERIODE     104  // F_PWM = 800 kHz (1.25us)
//#define  WS2812_LO_TIME         29  // 29 * 11.9ns = 0.34us
//#define  WS2812_HI_TIME         76  // 76 * 11.9ns = 0.90us
// STM32F1xx
#define  WS2812_TIM_PRESCALE       0
#define  WS2812_TIM_PERIODE       89  // F_PWM = 800 kHz (1.25us)
#define  WS2812_LO_TIME           28
#define  WS2812_HI_TIME           70

//--------------------------------------------------------------
// defines vom WS2812 (nicht abaendern)
//--------------------------------------------------------------
#define  WS2812_BIT_PER_LED    24  // 3*8bit pro LED
#define  WS2812_PAUSE_ANZ       2  // fuer Pause (2*30us)

#define  WS2812_TIMER_BUF_LEN    (WS2812_LED_MAX_ANZ+WS2812_PAUSE_ANZ)*WS2812_BIT_PER_LED

//--------------------------------------------------------------
// Globale Funktionen
//--------------------------------------------------------------
ErrorStatus UB_WS2812_Init(void);
void UB_WS2812_SetChannel(uint8_t ch);
void UB_WS2812_Refresh(void);
void UB_WS2812_RGB_2_HSV(WS2812_HSV_t hsv_col, WS2812_RGB_t *rgb_col);
void UB_WS2812_One_Led_RGB(uint32_t nr, WS2812_RGB_t rgb_col, uint8_t refresh);
void UB_WS2812_All_Led_RGB(WS2812_RGB_t rgb_col, uint8_t refresh);
void UB_WS2812_One_Led_HSV(uint32_t nr, WS2812_HSV_t hsv_col, uint8_t refresh);
void UB_WS2812_All_Led_HSV(WS2812_HSV_t hsv_col, uint8_t refresh);
void UB_WS2812_Shift_Left(void);
void UB_WS2812_Shift_Right(void);
void UB_WS2812_Rotate_Left(void);
void UB_WS2812_Rotate_Right(void);
ErrorStatus UB_WS2812_Check_Len(void);

void d_printArray( void );

//--------------------------------------------------------------
#endif // __STM32F4_UB_WS2812_H
